/***************************************************************************
 *   Copyright 1998,2000  Stephan Kulow <coolo@kde.org>                    *
 *   Copyright 2008 by Davide Bettio <davide.bettio@kdemail.net>           *
 *   Copyright 2009 by Glad Deschrijver <glad.deschrijver@gmail.com>       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "lunadialog.h"

#include <time.h>

#include <QGridLayout>
#include <QHBoxLayout>
#include <QKeyEvent>
#include <QLabel>
#include <QToolButton>
#include <QVBoxLayout>

//#include <KDebug>
#include <KGlobal>
#include <KIcon>
#include <KLocale>

#include <Plasma/Theme>

#include "lunacalc.h"

LunaDialog::LunaDialog(QWidget *parent, Qt::WindowFlags f)
    : Plasma::Dialog(parent, f)
{
	setWindowFlags(windowFlags() | Qt::WindowStaysOnTopHint | Qt::Popup);
//	setResizeHandleCorners(Plasma::Dialog::NorthEast);
	setResizeHandleCorners(Plasma::Dialog::All);

	QGridLayout *labelsLayout = new QGridLayout;
	QLabel *lastNewLabel = new QLabel(this);
	QLabel *firstQuarterLabel = new QLabel(this);
	QLabel *fullMoonLabel = new QLabel(this);
	QLabel *thirdQuarterLabel = new QLabel(this);
	QLabel *nextNewLabel = new QLabel(this);
	lastNewLabel->setText(i18n("Last new:"));
	firstQuarterLabel->setText(i18n("First quarter:"));
	fullMoonLabel->setText(i18n("Full moon:"));
	thirdQuarterLabel->setText(i18n("Third quarter:"));
	nextNewLabel->setText(i18n("Next new:"));
	labelsLayout->addWidget(lastNewLabel, 0, 0);
	labelsLayout->addWidget(firstQuarterLabel, 1, 0);
	labelsLayout->addWidget(fullMoonLabel, 2, 0);
	labelsLayout->addWidget(thirdQuarterLabel, 3, 0);
	labelsLayout->addWidget(nextNewLabel, 4, 0);
	m_lastNewLabel = new QLabel(this);
	m_firstQuarterLabel = new QLabel(this);
	m_fullMoonLabel = new QLabel(this);
	m_thirdQuarterLabel = new QLabel(this);
	m_nextNewLabel = new QLabel(this);
	labelsLayout->addWidget(m_lastNewLabel, 0, 1);
	labelsLayout->addWidget(m_firstQuarterLabel, 1, 1);
	labelsLayout->addWidget(m_fullMoonLabel, 2, 1);
	labelsLayout->addWidget(m_thirdQuarterLabel, 3, 1);
	labelsLayout->addWidget(m_nextNewLabel, 4, 1);
	labelsLayout->setColumnStretch(1, 1);

	QGridLayout *labelsLayout2 = new QGridLayout;
	QLabel *daysSinceLastNewLabel = new QLabel(this);
	daysSinceLastNewLabel->setText(i18n("Days since last new moon:"));
	labelsLayout2->addWidget(daysSinceLastNewLabel, 0, 0);
	m_daysSinceLastNewLabel = new QLabel(this);
	labelsLayout2->addWidget(m_daysSinceLastNewLabel, 0, 1);
	labelsLayout2->setColumnStretch(1, 1);

	QHBoxLayout *buttonsLayout = new QHBoxLayout;
	m_previousButton = new QToolButton(this);
	m_todayButton = new QToolButton(this);
	m_nextButton = new QToolButton(this);
	QPalette p = palette();
	p.setColor(QPalette::Button, Qt::transparent);
	m_previousButton->setPalette(p);
	m_todayButton->setPalette(p);
	m_nextButton->setPalette(p);
	m_previousButton->setIcon(KIcon("go-previous"));
	m_todayButton->setIcon(KIcon("go-jump-today"));
	m_nextButton->setIcon(KIcon("go-next"));
	connect(m_previousButton, SIGNAL(clicked()), this, SLOT(previousPhases()));
	connect(m_todayButton, SIGNAL(clicked()), this, SLOT(todayPhases()));
	connect(m_nextButton, SIGNAL(clicked()), this, SLOT(nextPhases()));
	buttonsLayout->addStretch();
	buttonsLayout->addWidget(m_previousButton);
	buttonsLayout->addWidget(m_todayButton);
	buttonsLayout->addWidget(m_nextButton);
	buttonsLayout->addStretch();

	QVBoxLayout *mainLayout = new QVBoxLayout;
	QWidget *labelsWidget = new QWidget(this);
	QWidget *labelsWidget2 = new QWidget(this);
	QWidget *buttonsWidget = new QWidget(this);
	labelsWidget->setLayout(labelsLayout);
	labelsWidget2->setLayout(labelsLayout2);
	buttonsWidget->setLayout(buttonsLayout);
	mainLayout->setSpacing(0);
	mainLayout->addWidget(labelsWidget);
	mainLayout->addWidget(labelsWidget2);
	mainLayout->addWidget(buttonsWidget);
	setLayout(mainLayout);

	m_lunaCalc = new LunaCalc(this);

	connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(updateColors()));
	updateColors();
}

LunaDialog::~LunaDialog()
{
}

void LunaDialog::keyPressEvent(QKeyEvent *event)
{
	switch (event->key())
	{
		case Qt::Key_Right:
		case Qt::Key_Down:
		{
			m_nextButton->animateClick();
			event->accept();
			break;
		}
		case Qt::Key_Left:
		case Qt::Key_Up:
		{
			m_previousButton->animateClick();
			event->accept();
			break;
		}
		case Qt::Key_Home:
		{
			m_todayButton->animateClick();
			event->accept();
			break;
		}
	}
	Plasma::Dialog::keyPressEvent(event);
}

void LunaDialog::wheelEvent(QWheelEvent *event)
{
	if (event->delta() > 0)
	{
		nextPhases();
	}
	else if (event->delta() < 0)
	{
		previousPhases();
	}
}

void LunaDialog::updateColors()
{
	QPalette pal = palette();
	pal.setColor(QPalette::Text, Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
	pal.setColor(QPalette::WindowText, Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
	pal.setColor(QPalette::Base, Qt::transparent);
	pal.setColor(QPalette::Window, Qt::transparent);
	//pal.setColor(QPalette::Button, Qt::transparent);
	pal.setColor(QPalette::ButtonText, Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
	setPalette(pal);
}

void LunaDialog::setDateTime(QDateTime dateTime)
{
	m_todayDateTime = dateTime;
	todayPhases();
}

void LunaDialog::setTodayDateTime(QDateTime dateTime)
{
	m_todayDateTime = dateTime;
}

void LunaDialog::calcStatus(time_t time, bool dateInCurrentMonth)
{
	QList<QDateTime> phases = m_lunaCalc->calcStatus(time);
	m_lastNewDateTime = phases.at(0);
	m_nextNewDateTime = phases.at(4);

//	kDebug() << "last new " << KGlobal::locale()->formatDateTime(phases.at(0));
	m_lastNewLabel->setText(KGlobal::locale()->formatDateTime(phases.at(0), KLocale::LongDate));

//	kDebug() << "first quarter " << KGlobal::locale()->formatDateTime(phases.at(1));
	m_firstQuarterLabel->setText(KGlobal::locale()->formatDateTime(phases.at(1), KLocale::LongDate));

//	kDebug() << "full moon " << KGlobal::locale()->formatDateTime(phases.at(2));
	m_fullMoonLabel->setText(KGlobal::locale()->formatDateTime(phases.at(2), KLocale::LongDate));

//	kDebug() << "third quarter " << KGlobal::locale()->formatDateTime(phases.at(3));
	m_thirdQuarterLabel->setText(KGlobal::locale()->formatDateTime(phases.at(3), KLocale::LongDate));

//	kDebug() << "next new " << KGlobal::locale()->formatDateTime(phases.at(4));
	m_nextNewLabel->setText(KGlobal::locale()->formatDateTime(phases.at(4), KLocale::LongDate));

	QDateTime currentTime;
	currentTime.setTime_t(time);
	if (dateInCurrentMonth) // this is always true when the dialog is shown for the first time
		m_daysSinceLastNewLabel->setText(QString::number(phases.at(0).daysTo(currentTime)));
}

void LunaDialog::previousPhases()
{
	calcStatus(m_lastNewDateTime.addDays(-1).toTime_t(), false);
}

void LunaDialog::todayPhases()
{
	calcStatus(m_todayDateTime.toTime_t(), true);
}

void LunaDialog::nextPhases()
{
	calcStatus(m_nextNewDateTime.addDays(1).toTime_t(), false);
}

#include "lunadialog.moc"
